package connectionspeedtest;

import java.net.URL;
import java.util.ResourceBundle;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.concurrent.Task;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.ComboBox;
import javafx.scene.control.Label;
import javafx.scene.control.ProgressBar;

public class FXMLDocumentController implements Initializable {
    @FXML
    private Label progressBarLabel;
    @FXML
    private Label resultLabel;
    @FXML
    private ProgressBar progressBar;
    @FXML
    private ComboBox testFileSizes;

    private SpeedTest speedTester;
    private double downloadSpeed, uploadSpeed;
    private long ping;
    
    @FXML
    private void handleStartButton(ActionEvent event){
        String testFileName = (String)testFileSizes.getSelectionModel().getSelectedItem();
        this.speedTester = new SpeedTest(testFileName);
        progressBar.setVisible(true);

        Task testInternetConnection = new Task<Void>() {
            @Override
            protected Void call() throws Exception {
                updateProgress(0, 3);
                updateTitle("Testing Download Speed");
                downloadSpeed = speedTester.measureDownloadSpeed();
                updateProgress(1, 3);
                updateTitle("Testing Upload Speed");
                uploadSpeed = speedTester.measureUploadSpeed();
                updateProgress(2, 3);
                updateTitle("Testing ping");
                ping = speedTester.measureLatency();
                updateMessage(createResultString());
                updateTitle("Finished");
                updateProgress(3, 3);
                return null;
            }
            
            private String createResultString(){
                String message = " Download: \t %.2f Kb/s \n "
                        + "Upload: \t\t %.2f Kb/s \n "
                        + "Ping: \t\t %d ms";
                return String.format(message, downloadSpeed, uploadSpeed, ping);
            }
        };

        bindProperties(testInternetConnection);
        runTestingThread(testInternetConnection);
    }
    
    private void bindProperties(Task task) {
        progressBar.progressProperty().bind(task.progressProperty());
        progressBarLabel.textProperty().bind(task.titleProperty());
        resultLabel.textProperty().bind(task.messageProperty());
    }
    
    private void runTestingThread(Task task) {
        Thread computationThread = new Thread(task);
        computationThread.setDaemon(true);
        computationThread.start();
    }
    
    @FXML
    private void handleResetButton(ActionEvent event){
        unbindProperties();
        progressBar.setVisible(false);
        resultLabel.setText(" ");
        progressBarLabel.setText("");
    }

    private void unbindProperties() {
        resultLabel.textProperty().unbind();
        progressBar.progressProperty().unbind();
        progressBarLabel.textProperty().unbind();
    }

    @Override
    public void initialize(URL url, ResourceBundle rb) {
        ObservableList<String> sizes = FXCollections.observableArrayList();
        for (int i = 0; i < 10; i++)
            sizes.add((i+1) + "MB");
        testFileSizes.setItems(sizes);
        testFileSizes.getSelectionModel().selectFirst();
    }    
    
}
