package connectionspeedtest;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URL;
import java.net.URLConnection;
import java.nio.channels.Channels;
import java.nio.channels.ReadableByteChannel;
import java.nio.channels.WritableByteChannel;

public class SpeedTest {
    private final String downloadSpeedTestFtpUrl;
    private final String uploadSpeedTestFtpUrl;
    private final String latencyTestUrl; 

    private double downloadSpeedInKilobytesPerSecond;
    private double uploadSpeedInKilobytesPerSecond;
    private long latencyInMiliseconds;
    
    private final String testFileName;
    
    public SpeedTest(){
        testFileName = "1MB";
        downloadSpeedTestFtpUrl = "ftp://jaroslavistok\\$jaroslavistok:zf9F2pi4m4sxYyD2td9icnqoWtjjjwtrDZ0FasfirS6QgNwqPcgGp1aakejF@ftp.gear.host/SpeedMeter/"+ testFileName;
        uploadSpeedTestFtpUrl = "ftp://jaroslavistok\\$jaroslavistok:zf9F2pi4m4sxYyD2td9icnqoWtjjjwtrDZ0FasfirS6QgNwqPcgGp1aakejF@ftp.gear.host/"+ testFileName;
        latencyTestUrl = "http://uniba.sk/";
    }
    
    public SpeedTest(String testFileName){
        this.testFileName = testFileName;
        downloadSpeedTestFtpUrl = "ftp://jaroslavistok\\$jaroslavistok:zf9F2pi4m4sxYyD2td9icnqoWtjjjwtrDZ0FasfirS6QgNwqPcgGp1aakejF@ftp.gear.host/SpeedMeter/"+ testFileName;
        uploadSpeedTestFtpUrl = "ftp://jaroslavistok\\$jaroslavistok:zf9F2pi4m4sxYyD2td9icnqoWtjjjwtrDZ0FasfirS6QgNwqPcgGp1aakejF@ftp.gear.host/"+ testFileName;
        latencyTestUrl = "http://uniba.sk/";
    }

    public double measureDownloadSpeed() {
        try {
            long downloandTimeInNanoSeconds = getDownloadTimeInNanoSeconds();
            File file = new File(testFileName);
            double sizeOfTheDownloadedFileInKilobytes = file.length() / 1000.0;
            double downloandTimeInSeconds = (double)downloandTimeInNanoSeconds / 1000000000.0;
            downloadSpeedInKilobytesPerSecond = sizeOfTheDownloadedFileInKilobytes / downloandTimeInSeconds;
        } catch (Exception exception){
            System.err.println(exception.getMessage());
        }
        return downloadSpeedInKilobytesPerSecond;
    }
    
    private long getDownloadTimeInNanoSeconds() throws IOException {
        URL website = new URL(downloadSpeedTestFtpUrl);
        long start, end;
        try (ReadableByteChannel channel = Channels.newChannel(website.openStream()); 
                FileOutputStream outputStream = new FileOutputStream(testFileName)) {
            start = System.nanoTime();
            outputStream.getChannel().transferFrom(channel, 0, Long.MAX_VALUE);
            end = System.nanoTime();
            outputStream.flush();
        }
        return end - start;
    }
    
    public double measureUploadSpeed(){
        try {
           File file = new File(testFileName);
           double sizeOfUploadedFileInKiloBytes = file.length() / 1000.0;
           long uploadTimeInNanoSeconds = getUploadTimeInNanoSeconds();
           double downloadTimeInSeconds = uploadTimeInNanoSeconds / 1000000000.0;
           uploadSpeedInKilobytesPerSecond = sizeOfUploadedFileInKiloBytes / downloadTimeInSeconds;
           file.delete();
        } catch (Exception exception){
            System.err.println(exception.getMessage());
        }
        return uploadSpeedInKilobytesPerSecond;
    }
    
    private long getUploadTimeInNanoSeconds() throws IOException {
        URL url = new URL(uploadSpeedTestFtpUrl);
        URLConnection connection = url.openConnection();
        long start, end;
        try (WritableByteChannel channel = Channels.newChannel(connection.getOutputStream()); 
                FileInputStream inputStream = new FileInputStream(testFileName)) {
            start = System.nanoTime();
            inputStream.getChannel().transferTo(0, Long.MAX_VALUE, channel);
            end = System.nanoTime();
        }
        return end - start;
    }
    
    public long measureLatency(){
        try {
            long latencyInNanoSeconds = getLatencyInNanoSeconds();
            latencyInMiliseconds = latencyInNanoSeconds / 1000000;
        } catch (Exception e){
            System.err.println(e.getMessage());
        }
        return latencyInMiliseconds;
    }
    
    private long getLatencyInNanoSeconds() throws IOException {
        URL url = new URL(latencyTestUrl);
        
        long startTime = System.nanoTime();
        url.openConnection().connect();
        long endTime = System.nanoTime();
        
        return endTime - startTime;
    }
}
