package com.company;

import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLConnection;
import java.nio.file.Files;
import java.util.stream.LongStream;

/**
 * Utility for measuring speed Internet
 * @author Ondej torc
 * @version 1.0
 */
public class InternetSpeed {
    private static final String PING_ADDRESS = "google.com";
    private static final String DOWNLOADING_ADDRESS = "http://ajax.googleapis.com/ajax/libs/jquery/1.8.3/jquery.js";
    private static final int BUFFER_SIZE = 4096;
    private static final int FILE_SIZE = 262;
    private static final String HOST = "ondrejstorc.moxo.cz/";
    private static final String USER = "u121790660.internetspeed";
    private static final String PASSWORD = "itnetwork";
    private static final String PATH = "./jquery.js";

    /**
     * Gets values of ping, download speed and upload speed. And show them
     * @param numberOfRepetitions number of times to perform the measurement, the higher the value the more accurate the results, but it takes longer
     */
    public static void GetCompleteData(int numberOfRepetitions){
        final boolean[] gettingInternetSpeed = {true};
        final String[] output = {""};
        Thread gettingSpeed = new Thread(() -> {
            output[0] = "Ping: " + InternetSpeed.GetPing(numberOfRepetitions) + " ms\n";
            System.out.println("Hodnota pingu zjistena");
            output[0] += "Download speed: " + InternetSpeed.DownloadSpeed(numberOfRepetitions) + " kb/s\n";
            System.out.println("Rychlost downloadu zjistena");
            output[0] += "Upload speed: " + InternetSpeed.UploadSpeed(numberOfRepetitions) + " kb/s\n";
            System.out.println("Rychlost uploadu zjistena");
            gettingInternetSpeed[0] = false;
            File file = new File(PATH);
            file.delete();
        }, "GettingSpeed");
        gettingSpeed.start();
        while (gettingInternetSpeed[0]){
            System.out.println("...Stale ziskavam data...");
            try {
                Thread.sleep(5000);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
        System.out.println("\n\n" + output[0]);
    }
    /**
     * Gets network latency to the server google.com
     * @param numberOfRepetitions number of times to perform the measurement, the higher the value the more accurate the results, but it takes longer
     * @return network latency in ms (milliseconds)
     */
    public static int GetPing(int numberOfRepetitions){
        return GetPing(numberOfRepetitions, PING_ADDRESS);
    }
    /**
     * Gets network latency to the specific server
     * @param numberOfRepetitions number of times to perform the measurement, the higher the value the more accurate the results, but it takes longer
     * @param pingAddress address of server
     * @return network latency in ms (milliseconds)
     */
    public static int GetPing(int numberOfRepetitions, String pingAddress){

        String cmd = "ping -n " + numberOfRepetitions + " " + pingAddress;
        try{
            Process myProcess = Runtime.getRuntime().exec(cmd);
            myProcess.waitFor();

            if(myProcess.exitValue() == 0) {
                String ping = convertStreamToString(myProcess.getInputStream());
                ping = ping.substring(ping.indexOf("Average = ")).substring(10);
                ping = ping.substring(0, ping.indexOf("ms"));
                return Integer.decode(ping);
            } else {
                return -1;
            }
        }catch (Exception ex){
            return -1;
        }
    }
    /**
     * Gets a download speed
     * @param numberOfRepetitions number of times to perform the measurement, the higher the value the more accurate the results, but it takes longer
     * @return download speed in kb/s
     */
    public static double DownloadSpeed(int numberOfRepetitions){
        long[] measuredTimes = new long[numberOfRepetitions];
        for (int i = 0; i < numberOfRepetitions; i++){
            long time = System.currentTimeMillis();
            try {
                downloadFile(DOWNLOADING_ADDRESS, "./");
            } catch (IOException e) {
                System.out.println("Nkde se nco pokazilo :(");
                break;
            }
            time = System.currentTimeMillis() - time;

            measuredTimes[i] = Math.round(FILE_SIZE / (time/1000.0));
        }
        return LongStream.of(measuredTimes).average().getAsDouble();
    }

    /**
     * Gets a upload speed
     * @param numberOfRepetitions number of times to perform the measurement, the higher the value the more accurate the results, but it takes longer
     * @return upload speed in kb/s
     */
    public static double UploadSpeed(int numberOfRepetitions){
        long[] measuredTimes = new long[numberOfRepetitions];
        for (int i = 0; i < numberOfRepetitions; i++){
            long time = System.currentTimeMillis();
            uploadFile(HOST, USER, PASSWORD, PATH, PATH);
            time = System.currentTimeMillis() - time;
            measuredTimes[i] = Math.round(FILE_SIZE / (time/1000.0));
        }
        return LongStream.of(measuredTimes).average().getAsDouble();
    }
    /**
     * Downloads a file from a URL
     * @param fileURL HTTP URL of the file to be downloaded
     * @param saveDir path of the directory to save the file
     * @throws IOException
     */
    private static void downloadFile(String fileURL, String saveDir)
            throws IOException {
        URL url = new URL(fileURL);
        HttpURLConnection httpConn = (HttpURLConnection) url.openConnection();
        int responseCode = httpConn.getResponseCode();

        // always check HTTP response code first
        if (responseCode == HttpURLConnection.HTTP_OK) {
            String fileName = "";
            String disposition = httpConn.getHeaderField("Content-Disposition");
            String contentType = httpConn.getContentType();
            int contentLength = httpConn.getContentLength();

            if (disposition != null) {
                // extracts file name from header field
                int index = disposition.indexOf("filename=");
                if (index > 0) {
                    fileName = disposition.substring(index + 10,
                            disposition.length() - 1);
                }
            } else {
                // extracts file name from URL
                fileName = fileURL.substring(fileURL.lastIndexOf("/") + 1,
                        fileURL.length());
            }
            InputStream inputStream = httpConn.getInputStream();
            String saveFilePath = saveDir + File.separator + fileName;

            // opens an output stream to save into file
            FileOutputStream outputStream = new FileOutputStream(saveFilePath);

            int bytesRead = -1;
            byte[] buffer = new byte[BUFFER_SIZE];
            while ((bytesRead = inputStream.read(buffer)) != -1) {
                outputStream.write(buffer, 0, bytesRead);
            }

            outputStream.close();
            inputStream.close();

        } else {
            System.out.println("No file to download. Server replied HTTP code: " + responseCode);
        }
        httpConn.disconnect();
    }
    /**
     * Uploads a file to a URL
     * @param host URL of FTP server
     * @param user User name
     * @param pass Password to FTP server
     * @param path Local file path
     * @param uploadPath Upload path
     */
    public static void uploadFile(String host, String user, String pass, String path, String uploadPath) {
        String ftpUrl = "ftp://%s:%s@%s/%s;type=i";

        ftpUrl = String.format(ftpUrl, user, pass, host, path);

        try {
            URL url = new URL(ftpUrl);
            URLConnection conn = url.openConnection();
            OutputStream outputStream = conn.getOutputStream();
            FileInputStream inputStream = new FileInputStream(uploadPath);

            byte[] buffer = new byte[BUFFER_SIZE];
            int bytesRead = -1;
            while ((bytesRead = inputStream.read(buffer)) != -1) {
                outputStream.write(buffer, 0, bytesRead);
            }

            inputStream.close();
            outputStream.close();

        } catch (IOException ex) {
            ex.printStackTrace();
        }
    }

    /**
     * Returns string fromm {@link InputStream}
     * @param inputStream from which will be string get
     * @return content of input stream represented as {@link String}
     */
    private static String convertStreamToString(InputStream inputStream) {
        java.util.Scanner s = new java.util.Scanner(inputStream).useDelimiter("\\A");
        return s.hasNext() ? s.next() : "";
    }
}
