package cz.matejkripner;

import cz.matejkripner.user.ConnectionQuality;

/**
 * Set of static support methods.
 *
 * @author Matej Kripner <kripnermatej@gmail.com>
 * @version 1.0
 */
public class Util {

    private Util() {
        // exists only to defeat initialization
    }

    /**
     * Join the given thread at any cost.
     *
     * @param thread to be joined
     */
    public static void join(Thread thread) {
        try {
            thread.join();
        } catch (InterruptedException e) {
            join(thread);
        }
    }

    private static final String MBps = "MB/s";
    private static final String KBps = "KB/s";
    private static final String Bps = "B/s";
    /**
     * If the connection is lower than this, we say, that there is no connection.
     */
    private static final int MIN_BPS = 20;

    /**
     * Returns message for the user based on given read/write speed.
     *
     * @param Bps read/write speed
     * @return message for the user about the speed
     */
    public static String getIOSPeed(int Bps) {
        if (Bps < MIN_BPS) return ConnectionQuality.NO_CONNECTION.toString(); // no connection
        if (Bps >= 1_000_000) return String.format("%.3f", (double) Bps / 1_000_000) + MBps;
        if (Bps >= 1_000) return String.format("%.1f", (double) Bps / 1_000) + KBps;
        return Bps + Util.Bps;
    }
}
