package cz.matejkripner.services;

import javafx.beans.property.DoubleProperty;
import javafx.beans.property.SimpleDoubleProperty;

/**
 * Base class for all services in the application. Define main {@link #measure()} method.
 *
 * @author Matej Kripner <kripnermatej@gmail.com>
 * @version 1.0
 */
public abstract class Service {

    private final DoubleProperty progress;

    public Service() {
        this.progress = new SimpleDoubleProperty();
    }

    /**
     * Do the measurement and return a measured result.
     *
     * @return a measured result
     */
    public abstract Result measure();

    protected void setProgress(double progress) {
        this.progress.set(progress);
    }

    protected void addProgress(double add) {
        setProgress(progress.get() + add < 1 ? progress.get() + add : 0.99);
    }

    /**
     * Call this after completing the measurement.
     */
    protected void done() {
        setProgress(1);
    }

    /**
     * Call this before starting the measurement.
     */
    protected void begin() {
        setProgress(0);
    }

    public DoubleProperty progressProperty() {
        return progress;
    }
}
