package cz.matejkripner.services.download;

import cz.matejkripner.services.MeasurementException;
import cz.matejkripner.services.Result;
import cz.matejkripner.services.Service;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.time.LocalTime;
import java.time.temporal.ChronoUnit;

/**
 * Service for download speed measuring.
 *
 * @author Matej Kripner <kripnermatej@gmail.com>
 * @version 1.0
 */
public class DownloadService extends Service {
    /**
     * Domain of file to be downloaded
     */
    private static final String DOMAIN = "http://www.iloveutest.matejkripner.cz";
    /**
     * Path of file to be downloaded
     */
    private static final String FILE_PATH = "/testFile.txt";

    /**
     * {@inheritDoc}
     */
    @Override
    public Result measure() {
        begin();
        // get URL of the test file
        URL URL;
        try {
            URL domain = new URL(DOMAIN);
            URL = new URL(domain, FILE_PATH);
        } catch (MalformedURLException e) {
            throw new MeasurementException("URL cannot be created!", false, e);
        }
        File file = new File(URL.getPath());
        long length = file.length(); // length of the file
        LocalTime start = LocalTime.now();
        // read the file (results are not saved)
        try (InputStreamReader reader = new InputStreamReader(new FileInputStream(file))) {
            while (reader.read() != -1) ;
        } catch (IOException e) {
            throw new MeasurementException("Test file cannot be downloaded!", false, e);
        }
        // get time of the operation
        double time = (double) start.until(LocalTime.now(), ChronoUnit.MILLIS) / 1000;
        done();
        return new DownloadResult((int) (length / time), true);
    }
}
