package cz.matejkripner.services.ping;

import cz.matejkripner.services.MeasurementException;
import cz.matejkripner.services.ProcessReader;
import cz.matejkripner.services.Service;

import java.io.IOException;

/**
 * Service for pinging (latency measuring).
 *
 * @author Matej Kripner <kripnermatej@gmail.com>
 * @version 1.0
 */
public class PingService extends Service {

    private static final String DEFAULT_HOST = "216.58.192.46"; // google.com

    /**
     * Host to ping to. If null, {@link #DEFAULT_HOST} is used.
     */
    private String host;

    public void setHost(String host) {
        this.host = host;
    }


    /**
     * {@inheritDoc}
     */
    @Override
    public PingResult measure() {
        begin();
        String src = ping();
        PingResult pingResult = new PingResult(src);
        done();
        return pingResult;
    }

    /**
     * Ping and returns the response.
     *
     * @return the response of pinging.
     */
    private String ping() {
        Process process;
        try {
            process = Runtime.getRuntime().exec("ping " + (host == null || host.isEmpty() ? DEFAULT_HOST : host));
        } catch (IOException e) {
            throw new MeasurementException("Everything is wrong, we can't ping!");
        }
        return ProcessReader.getReader().read(process);
    }
}
