package cz.matejkripner.services.upload;

import cz.matejkripner.Util;
import cz.matejkripner.services.MeasurementException;
import cz.matejkripner.services.Result;
import cz.matejkripner.user.ConnectionQuality;

/**
 * Result of a {@link UploadResult} measuring. Mainly contains the upload speed.
 *
 * @author Matej Kripner <kripnermatej@gmail.com>
 * @version 1.0
 */
public class UploadResult extends Result {

    private final boolean ok;
    /**
     * The upload speed (in bytes per second).
     */
    private final int Bps;

    public UploadResult(int bps, boolean ok) {
        this.ok = ok;
        Bps = bps;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Result merge(Result... results) {
        return this.mergeHelp(UploadResult::getBps, Bps -> new UploadResult(Bps, true), results);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean everythingOK() {
        return ok;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String toHumanText() throws MeasurementException {
        String value = Util.getIOSPeed(Bps);
        if (value.equals(ConnectionQuality.NO_CONNECTION.toString()))
            throw new MeasurementException("Your upload speed is too slow to be measured");
        return value;
    }

    public int getBps() {
        return Bps;
    }
}
