package cz.matejkripner.services.upload;

import cz.matejkripner.services.MeasurementException;
import cz.matejkripner.services.Result;
import cz.matejkripner.services.Service;

import java.io.BufferedOutputStream;
import java.io.IOException;
import java.net.URL;
import java.net.URLConnection;
import java.time.LocalTime;
import java.time.temporal.ChronoUnit;
import java.util.Random;

/**
 * Service for upload speed measuring.
 *
 * @author Matej Kripner <kripnermatej@gmail.com>
 * @version 1.0
 */
public class UploadService extends Service {

    private static final String FTP_SERVER = "?";
    private static final String USER = "?";
    private static final String PASSWORD = "?";
    private static final int BYTES = 10_000_000; // bytes to upload

    @Override
    public Result measure() {
        begin();
        LocalTime start = LocalTime.now();
        String fileName = "testFile" + (new Random()).nextInt();
        try {
            upload(fileName);
        } catch (IOException e) {
            throw new MeasurementException("Test file couldn't be uploaded", false, e);
        }
        int time = (int) start.until(LocalTime.now(), ChronoUnit.SECONDS);
        return new UploadResult(BYTES / time, true);
    }

    public void upload(String fileName) throws IOException {
        if (fileName == null) throw new IllegalArgumentException("file name has to be not-null");

        String urlString = getFtpUrl(fileName);

        URL url = new URL(urlString);
        URLConnection urlConnection = url.openConnection();
        try (BufferedOutputStream bos = new BufferedOutputStream(urlConnection.getOutputStream())) {
            for (int i = 0; i < BYTES; i++) {
                bos.write(354); // random integer
            }
        }
    }

    private String getFtpUrl(String fileName) {
        return "ftp://" + USER + ':' + PASSWORD + '@' + FTP_SERVER + '/' + fileName + ";type=a";
    }
}
