package cz.matejkripner.user;

import java.util.Arrays;
import java.util.Optional;

/**
 * Base grades of the connection quality.
 *
 * @author Matej Kripner <kripnermatej@gmail.com>
 * @version 1.0
 */
public enum ConnectionQuality {
    VERY_GOOD(0, 50, "Very good", Color.VERY_GOOD.get()),
    GOOD(51, 90, "Good", Color.GOOD.get()),
    STANDARD(91, 120, "Standard", Color.NEUTRAL.get()),
    BAD(121, 180, "Bad", Color.BAD.get()),
    VERY_BAD(181, Integer.MAX_VALUE, "Very bad", Color.VERY_BAD.get()),
    NO_CONNECTION("No connection!"),
    NOT_MEASURED("Not measured");

    private final int latencyMin;
    private final int latencyMax;

    /**
     * Human representation of the grade.
     */
    private final String display;

    private final javafx.scene.paint.Color color;

    ConnectionQuality(int latencyMin, int latencyMax, String display, javafx.scene.paint.Color color) {
        this.latencyMin = latencyMin;
        this.latencyMax = latencyMax;
        this.display = display;
        this.color = color;
    }

    ConnectionQuality(javafx.scene.paint.Color color, String display) {
        this(-1, -1, display, color);
    }

    ConnectionQuality(String display) { // no bounds specified
        this(javafx.scene.paint.Color.BLACK, display);
    }

    public javafx.scene.paint.Color getColor() {
        return color;
    }

    /**
     * Get the grade of the connection quality from given the net latency.
     *
     * @param latency of the result grade
     * @return grade of the connection quality from the given net latency
     */
    public static ConnectionQuality get(int latency) {
        Optional<ConnectionQuality> connectionQuality = Arrays
                .stream(values())
                .filter(quality -> latency >= quality.latencyMin && latency <= quality.latencyMax)
                .findFirst();
        return connectionQuality.isPresent() ? connectionQuality.get() : null;
    }

    @Override
    public String toString() {
        return display;
    }

}
