package cz.matejkripner.user;

import cz.matejkripner.Measurement;
import javafx.application.Platform;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.ProgressIndicator;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.VBox;

import java.util.Arrays;

/**
 * Class used to build scene and communicate with the {@link Measurement} instances.
 *
 * @author Matej Kripner <kripnermatej@gmail.com>
 * @version 1.0
 */
public class MainWindow {

    /**
     * Warning for the user.
     */
    private Label warning;

    private ProgressIndicator latencyProgressIndicator;
    private ProgressIndicator downloadProgressIndicator;
//    private ProgressIndicator uploadProgressIndicator;

    private Label latencyInformation;
    private Label downloadInformation;
    private Label uploadInformation;

    private Button button;

    private final Scene scene;

    public MainWindow() {
        scene = initScene();
    }

    /**
     * Main method - create and return the scene.
     *
     * @return prepared scene
     */
    private Scene initScene() {
        VBox box = initBox(); // main container
        GridPane grid = initGrid(); // grid pane for the values
        warning = initWarningLabel();

        button = initMeasurementButton();

        latencyProgressIndicator = new ProgressIndicator();
        downloadProgressIndicator = new ProgressIndicator();
//        uploadProgressIndicator = new ProgressIndicator();

        latencyInformation = new Label();
        downloadInformation = new Label();
        uploadInformation = new Label();
        uploadInformation.setText("Please see README.txt");

        String[] labels = {"Latency:", "Download speed:", "Upload speed:"};
        ProgressIndicator[] progressIndicators = {latencyProgressIndicator, downloadProgressIndicator};
        Arrays.stream(progressIndicators).forEach(progressIndicator -> progressIndicator.setVisible(false)); // invisible all progress indicators
        Label[] information = {latencyInformation, downloadInformation, uploadInformation};
        // add to the grid
        grid.addColumn(0, Arrays.stream(labels).map(Label::new).toArray(Label[]::new)); // converts all Strings to Labels
        grid.addColumn(1, progressIndicators);
        grid.addColumn(2, information);

        // main button
        button.setOnAction(event -> new Thread(this::work).start());

        box.getChildren().addAll(initWarningLabel(), grid, button);
        return new Scene(box);
    }

    /**
     * Do all the measurement stuff.
     */
    private void work() {
        Platform.runLater(() -> button.setDisable(true)); // disable main button to have only 1 action to proceed at time

        Values values = new Values();
        // set progress indicators and labels for results writing
        values.setLatency(new Values.Property(latencyProgressIndicator, latencyInformation));
        values.setDownloadSpeed(new Values.Property(downloadProgressIndicator, downloadInformation));
//        values.setUploadSpeed(new Values.Property(uploadProgressIndicator, uploadInformation));
        values.setWarning(warning);
        Measurement m = new Measurement(values);

        m.measure(); // do the work

        Platform.runLater(() -> button.setDisable(false));
    }

//    public void localError(Service service, Values.Property property) {
//        Button tryAgainButton = initTryAgainButton();
//        tryAgainButton.setOnAction(event -> {
//
//        });
//    }

    private Label initWarningLabel() {
        Label warningLabel = new Label();
        warningLabel.setTextFill(cz.matejkripner.user.Color.ERROR.get());
        String warning = Measurement.warning();
        if (warning != null) warningLabel.setText(warning);
        return warningLabel;
    }

    private GridPane initGrid() {
        GridPane grid = new GridPane();
        grid.setAlignment(Pos.CENTER);
        grid.setPadding(new Insets(20));
        grid.setHgap(30);
        grid.setVgap(15);
        return grid;
    }

    private VBox initBox() {
        VBox box = new VBox();
        box.setSpacing(10);
        box.setPadding(new Insets(60, 60, 60, 60));
        box.setAlignment(Pos.CENTER);
        return box;
    }

    private Button initMeasurementButton() {
        Button button = new Button("Measure");
        return button;
    }

//    private Button initTryAgainButton(Service service, Values.Property property) {
//        Button button = new Button("Try it again");
//        return button;
//    }

    public Scene getScene() {
        return scene;
    }

}
