package cz.matejkripner.user;

import javafx.application.Platform;
import javafx.scene.control.Label;
import javafx.scene.control.ProgressIndicator;

import java.util.function.Consumer;

/**
 * Container for all values that should the user see.
 *
 * @author Matej Kripner <kripnermatej@gmail.com>
 * @version 1.0
 */
public class Values {

    /**
     * Warning for the user.
     */
    private Label warning;

    private Property latency;
    private Property downloadSpeed;
//    private Property uploadSpeed;


    public Property latency() {
        return latency;
    }

    public Property downloadSpeed() {
        return downloadSpeed;
    }

//    public Property uploadSpeed() {
//        return uploadSpeed;
//    }


    public void setLatency(Property latency) {
        this.latency = latency;
    }

    public void setDownloadSpeed(Property downloadSpeed) {
        this.downloadSpeed = downloadSpeed;
    }

//    public void setUploadSpeed(Property uploadSpeed) {
//        this.uploadSpeed = uploadSpeed;
//    }


    public void setWarning(Label warningLabel) {
        warning = warningLabel;
    }

    public void setWarning(String warning) {
        this.warning.setText(warning);
    }

    /**
     * Do the action for each property.
     *
     * @param action the action
     */
    public void forEach(Consumer<Property> action) {
        action.accept(latency);
        action.accept(downloadSpeed);
//        action.accept(uploadSpeed);
    }


    public static class Property {

        /**
         * String to be appended after value that is still changing (if the running flag is true)
         */
        public static final String WAITING = " ...";

        /**
         * If the measurement that fill this property is still running.
         */
        private boolean running;

        private final ProgressIndicator progressIndicator;
        private final Label information;

        /**
         * Last information to the {@link #information} (without {@link #WAITING}) string.
         */
        private String lastInformation;

        public Property(ProgressIndicator progressIndicator, Label information) {
            this.progressIndicator = progressIndicator;
            this.information = information;
            lastInformation = "";
            running = false;
        }

        public void setProgress(double progress) {
            perform(() -> progressIndicator.setProgress(progress));
        }

        public void addProgress(double add) {
            setProgress(getProgress() + add > 1 ? 0.99 : getProgress() + add);
        }

        public double getProgress() {
            return progressIndicator.getProgress();
        }

        public void setInformation(String information, javafx.scene.paint.Color color) {
            lastInformation = information;
            perform(() -> {
                this.information.setText(running ? lastInformation + WAITING : lastInformation);
                this.information.setTextFill(color);
            });
        }

        public void setInformation(String information) {
            setInformation(information, javafx.scene.paint.Color.BLACK);
        }

        public void setInformation(ConnectionQuality connectionQuality) {
            setInformation(connectionQuality.toString(), connectionQuality.getColor());
        }

        /**
         * Perform an action that has to be performed only in FXApplicationThread.
         *
         * @param action the action
         */
        private void perform(Runnable action) {
            if (Platform.isFxApplicationThread()) action.run();
            else Platform.runLater(action);
        }


        public boolean isRunning() {
            return running;
        }

        public void setRunning(boolean running) {
            this.running = running;
            setInformation(lastInformation);
            setProgress(running ? 0 : 1); // started / ended
            perform(() -> progressIndicator.setVisible(true)); // visible the progress indicator
        }

    }
}
