import java.io.*;
import java.net.*;
//import java.nio.*;

public class Tests {
	
	public float[] downloadSpeeds;
	public float[] uploadSpeeds;
	public float[] pingTests;
	
	public Tests(byte testAccuracy)
	{
		downloadSpeeds = new float[testAccuracy];
		uploadSpeeds = new float[testAccuracy];
		pingTests = new float[testAccuracy];
	}
	
	public String DownloadTest(String sizeOfTestedFile, int currentIteration) throws IOException
	{
		int size;
	    URL url;
	    if (sizeOfTestedFile.equals("small"))
	    {
	    	url = new URL("http://www.westudio.6f.sk/d_small_file.data");
	    	size = 1048576;
	    }
	    else if (sizeOfTestedFile.equals("medium"))
	    {
	    	url = new URL("http://www.westudio.6f.sk/d_medium_file.data");
	    	size = 2097152;
	    }
	    else
	    {
	    	url = new URL("http://www.westudio.6f.sk/d_big_file.data");
	    	size = 4194304;
	    }
	    URLConnection urlConnection = url.openConnection();
	    
	    //Streams
	    InputStream inputStream = urlConnection.getInputStream();
	    BufferedInputStream bufferedInputStream = new BufferedInputStream(inputStream);
	    ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
	    
	    int wasRead = 0;
	    byte[] buffer = new byte[1024];
	    
	    long startTime = System.currentTimeMillis();
	    while ((wasRead = bufferedInputStream.read(buffer)) != -1)
	    {
	    	byteArrayOutputStream.write(buffer, 0, wasRead);
	    }
	    long endTime = System.currentTimeMillis();
	    
	    float speed = ((float)size / 1024f) / ((endTime - startTime) / 1000);
	    downloadSpeeds[currentIteration] = speed;
	    
	    if (speed < 1000)
	    	return String.format("%.2f Kb/s", Math.round(speed * 100.0f) / 100.0f);
	    else
	    	return String.format("%.2f Mb/s", Math.round((speed / 1024) * 100.0f) / 100.0f);
	}
	
	public String UploadTest(String sizeOfTestedFile, int currentIteration) throws IOException
	{   
		int size;
	    if (sizeOfTestedFile.equals("small"))
	    {
	    	size = 524288;
	    }
	    else if (sizeOfTestedFile.equals("medium"))
	    {
	    	size = 1048576;
	    }
	    else
	    {
	    	size = 2097152;
	    }
	    
	    Socket socket = new Socket("www.westudio.6f.sk", 80);
	    String file = new String(new byte[size]);
	    
	    BufferedWriter bufferedWriter = new BufferedWriter(new OutputStreamWriter(socket.getOutputStream(), "UTF8"));
	    
	    bufferedWriter.write("POST " + "http://www.westudio.6f.sk" + " HTTP/1.0\r\n");
	    bufferedWriter.write("Content-Length: " + file.length() + "\r\n");
	    bufferedWriter.write("Content-Type: plain/text\r\n");
	    bufferedWriter.write("\r\n");
	    
	    long startTime = System.currentTimeMillis();
	    bufferedWriter.write(file);
	    long endTime = System.currentTimeMillis();
	    
	    bufferedWriter.flush();
	    socket.close();
	    
	    float speed = ((float)size / 1024f) / ((endTime - startTime) / 1000);
	    uploadSpeeds[currentIteration] = speed;
	    
	    if (speed < 1000)
	    	return String.format("%.2f Kb/s", Math.round(speed * 100.0f) / 100.0f);
	    else
	    	return String.format("%.2f Mb/s", Math.round((speed / 1024) * 100.0f) / 100.0f);
	}
	
	public int PingTest(int currentIteration) throws InterruptedException, IOException
	{
		String command = "";
        if(System.getProperty("os.name").startsWith("Windows"))
        	command = "ping -n 2 88.86.120.210"; // Windows
        else
        	command = "ping -c 2 88.86.120.210"; // Linux, MAC OSX

        Process process = Runtime.getRuntime().exec(command);
        process.waitFor();
        
        BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(process.getInputStream()));
        String output = "";
        
        while (bufferedReader.readLine() != null)
        	output += bufferedReader.readLine();

        int speed = Integer.parseInt(output.substring(output.indexOf("time=") + 5, output.indexOf("ms")));
        pingTests[currentIteration] = speed;
        return speed;
	}
}
