package brambora;

import java.io.IOException;
import java.net.URL;
import java.text.DecimalFormat;
import java.util.ResourceBundle;
import javafx.application.Platform;
import javafx.beans.value.ObservableValue;
import javafx.concurrent.Service;
import javafx.concurrent.Task;
import javafx.concurrent.WorkerStateEvent;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.ProgressIndicator;
import javafx.scene.control.TextArea;

/**
 * @author brambora
 */
public class FXMLDocumentController implements Initializable {

    private static final int TEST_COUNT = 10;
    private static final int STRIP_SIZE = 1;
    private final DecimalFormat formater = new DecimalFormat("#.##");
    private Speedtestdotnet tester;
    
    private Service<Void> downloadTest;
    private Service<Void> uploadTest;
    private Service<Void> latencyTest;

    private boolean someTestRunning;
    
    @FXML
    private TextArea log;

    @FXML
    private ProgressIndicator progress;

    @FXML
    private void runDownloadTest(ActionEvent event) {
        runTest(downloadTest);
    }

    @FXML
    private void runUploadTest(ActionEvent event) {
        runTest(uploadTest);
    }

    @FXML
    private void runLatencyTest(ActionEvent event) {
        runTest(latencyTest);
    }

    private void runTest(Service test) {
        if (!someTestRunning) {
            test.reset();
            test.start();
        }
    }

    @FXML
    private void runCompleteTest(ActionEvent event) {
        runTest(downloadTest);
        downloadTest.setOnSucceeded((WorkerStateEvent e) -> {
            runTest(uploadTest);
        });
        uploadTest.setOnSucceeded((WorkerStateEvent e) -> {
            runTest(latencyTest);
        });

        latencyTest.setOnSucceeded((WorkerStateEvent e) -> {
            downloadTest.setOnSucceeded(null);
            uploadTest.setOnSucceeded(null);
            latencyTest.setOnSucceeded(null);
        });
    }

    private float proceedTest(ThrowableBiFunction<Integer, Integer, Float> test) {
        someTestRunning = true;
        float finalResult = 0;  
        
        try {           
            finalResult = test.apply(TEST_COUNT, STRIP_SIZE);
        } catch (IOException ex) {
            Platform.runLater(() -> {
                log.appendText("Error occured: " + ex.getLocalizedMessage() + "\n");
            });
        }

        someTestRunning = false;

        return finalResult;
    }

    @Override
    public void initialize(URL url, ResourceBundle rb) {
        tester = new Speedtestdotnet();

        tester.getProgressProperty().addListener((ObservableValue<? extends Number> observable, Number oldValue, Number newValue) -> {
            Platform.runLater(() -> {
                progress.progressProperty().set((double) ((float)newValue));
            });
        });
        
        downloadTest = new Service<Void>() {
            @Override
            protected Task<Void> createTask() {
                return new Task<Void>() {
                    @Override
                    protected Void call() {
                        float downloadSpeed = proceedTest(tester::testDownloadSpeed);
                        float bytesPerSecond = downloadSpeed / TEST_COUNT;

                        Platform.runLater(() -> {
                            log.appendText("Download Speed: " + numFormat(Converter.BytesToMebiBytes(bytesPerSecond)) + " MB/s");
                            log.appendText(" (" + numFormat(Converter.BytesToMegaBits(bytesPerSecond)) + " mbps) \n");
                        });

                        return null;
                    }
                };
            }
        };
        uploadTest = new Service<Void>() {
            @Override
            protected Task<Void> createTask() {
                return new Task<Void>() {
                    @Override
                    protected Void call() {
                        float uploadSpeed = proceedTest(tester::testUploadSpeed);
                        float bytesPerSecond = uploadSpeed / TEST_COUNT;

                        Platform.runLater(() -> {
                            log.appendText("Upload Speed: " + numFormat(Converter.BytesToMebiBytes(bytesPerSecond)) + " MB/s");
                            log.appendText(" (" + numFormat(Converter.BytesToMegaBits(bytesPerSecond)) + " mbps) \n");
                        });

                        return null;
                    }
                };
            }
        };
        latencyTest = new Service<Void>() {
            @Override
            protected Task<Void> createTask() {
                return new Task<Void>() {
                    @Override
                    protected Void call() {
                        float latency = proceedTest(tester::testLatency);
                        float latencyInMillis = latency / TEST_COUNT;

                        Platform.runLater(() -> {
                            log.appendText("Ping: " + numFormat(latencyInMillis) + " ms \n");
                        });

                        return null;
                    }
                };
            }
        };
    }

    private String numFormat(float num) {
        return formater.format(num);
    }

}